import { checkSession } from './session.js';
import { accessOrCreateUser, logoutUser as authLogout } from './auth.js';
import { renderExplorationPage, initExplorationSystem } from './explore.js';
import { renderInventoryPage, initInventorySystem } from './inventory.js';
import { renderProfilePage, initProfileSystem } from './profile.js';
import { renderEquipmentPage, initEquipmentSystem } from './equipment.js';
import { renderVillagePage, initVillageSystem } from './village.js';
import { renderSaveLoadPage, initSaveLoadSystem } from './saveLoad.js';
import { renderEncyclopediaPage, initEncyclopediaSystem } from './encyclopedia.js';
import { displayAlert } from './utils.js';
import type { User } from './types.js';
import { translate } from './localization.js';
import { loadUserData } from './data.js';
import { getIcon } from './icons.js';

const appElement = document.getElementById('app') as HTMLElement;
let currentUser: User | null = null;

// XP thresholds significantly increased for longer progression
const XP_THRESHOLDS = [0, 150, 400, 800, 1500, 2800, 4500, 7000, 10000, 14000, 19000, 25000, 32000, 40000, 50000, 65000, 80000, 100000, 125000, 150000];

export function getXpThresholds() {
    return XP_THRESHOLDS;
}

export function setCurrentUser(user: User | null) {
    currentUser = user;
}

export function getCurrentGameUser(): User | null {
    return currentUser;
}

function renderLoginPage() {
    appElement.innerHTML = `
        <div class="flex items-center justify-center min-h-screen bg-gray-900 p-4">
            <div class="card w-full max-w-lg shadow-2xl">
                <div class="text-center mb-8">
                    <h1 class="text-4xl font-bold text-primary-500">${translate('game_title_main')}</h1>
                </div>
                <h2 class="text-3xl font-semibold text-center mb-6 text-gray-200">${translate('login_title_simplified')}</h2>
                <form id="loginForm" class="space-y-6">
                    <div>
                        <label for="loginUsername" class="block mb-2 text-sm font-medium text-gray-300 w-full text-left">${translate('login_username_label')}</label>
                        <input type="text" id="loginUsername" name="username" class="input-field focus:ring-2 focus:ring-primary-500 w-full" required autocomplete="username" minlength="3">
                    </div>
                    <button type="submit" class="btn btn-primary w-full py-3 text-lg">${translate('login_button_simplified')}</button>
                </form>
                <div id="loginError" class="mt-4 text-red-400 text-sm text-center"></div>
            </div>
        </div>
    `;

    document.getElementById('loginForm')?.addEventListener('submit', async (e) => {
        e.preventDefault();
        const username = (document.getElementById('loginUsername') as HTMLInputElement).value;
        const errorDiv = document.getElementById('loginError') as HTMLElement;
        errorDiv.textContent = '';

        if (username.length < 3) {
            errorDiv.textContent = translate('username_min_length', 3);
            return;
        }

        try {
            const user = await accessOrCreateUser(username);
            if (user) {
                navigateToGame();
            } else {
                errorDiv.textContent = translate('login_unexpected_error');
            }
        } catch (error) {
            console.error('Access/Create failed:', error);
            errorDiv.textContent = (error as Error).message || translate('login_unexpected_error');
        }
    });
}


function renderGamePage() {
    if (!currentUser) {
        navigateToLogin();
        return;
    }

    const currentLevelXP = XP_THRESHOLDS[currentUser.level -1 ] || 0;
    const nextLevelXP = XP_THRESHOLDS[currentUser.level];

    let xpProgressInLevel = 0;
    let xpNeededForLevelUp = Infinity;
    let xpBarPercentage = 100;

    if (currentUser.level < XP_THRESHOLDS.length && nextLevelXP !== undefined) {
        xpProgressInLevel = currentUser.xp - currentLevelXP;
        xpNeededForLevelUp = nextLevelXP - currentLevelXP;
        xpBarPercentage = xpNeededForLevelUp > 0 ? (xpProgressInLevel / xpNeededForLevelUp) * 100 : 0;
    } else {
        xpProgressInLevel = currentUser.xp - currentLevelXP;
        xpNeededForLevelUp = 0;
    }

    xpBarPercentage = Math.max(0, Math.min(100, xpBarPercentage));

    appElement.innerHTML = `
        <div class="flex flex-col min-h-screen">
            <nav class="bg-gray-800 shadow-md sticky top-0 z-50">
                <div class="container mx-auto px-4">
                    <div class="flex items-center justify-between h-16">
                        <div class="flex items-center">
                            <img src="${currentUser.avatar}" alt="${currentUser.username}'s Avatar" class="w-10 h-10 rounded-full mr-3 border-2 border-primary-500 object-cover">
                            <div>
                                <span class="text-lg font-semibold text-white">${currentUser.username}</span>
                                <div class="text-xs text-gray-400 flex space-x-2">
                                    <span>${translate('level_abbreviation')}: <span id="userLevel">${currentUser.level}</span></span>
                                    <span>${translate('xp_abbreviation')}: <span id="userXP">${currentUser.xp.toLocaleString()}</span><span id="xpToNextLevel"> / ${nextLevelXP !== undefined ? nextLevelXP.toLocaleString() : translate('max_level_indicator')}</span></span>
                                    <span class="flex items-center">${getIcon('goldCoin', 'w-3 h-3 mr-1 text-yellow-400')}<span id="userGold">${currentUser.gold.toLocaleString()}</span></span>
                                </div>
                                <div class="w-32 bg-gray-700 rounded-full h-1.5 mt-1" aria-label="XP Progress">
                                    <div id="xpBar" class="bg-primary-500 h-1.5 rounded-full" style="width: ${xpBarPercentage}%" role="progressbar" aria-valuenow="${currentUser.xp}" aria-valuemin="${currentLevelXP}" aria-valuemax="${nextLevelXP || currentUser.xp}"></div>
                                </div>
                            </div>
                        </div>
                        <div class="flex items-center space-x-3">
                           <button id="logoutButton" class="btn btn-secondary text-sm">${translate('logout_button')}</button>
                           <div class="relative">
                               <button id="menuButton" aria-label="${translate('menu_open_nav')}" aria-haspopup="true" aria-expanded="false" class="p-1.5 rounded-md hover:bg-gray-700 focus:outline-none focus:ring-2 focus:ring-primary-500">
                                   ${getIcon('menuBars', 'w-5 h-5 text-white')}
                               </button>
                               <div id="navDropdown" class="absolute right-0 mt-2 w-52 bg-gray-700 rounded-md shadow-lg py-1 z-20 hidden ring-1 ring-black ring-opacity-5" role="menu" aria-orientation="vertical" aria-labelledby="menuButton">
                                   <a href="#" data-page="explore" class="nav-link flex items-center px-3 py-1.5 text-xs text-gray-200 hover:bg-primary-600 hover:text-white" role="menuitem">${getIcon('explore', 'w-4 h-4 mr-2')}${translate('menu_explore')}</a>
                                   <a href="#" data-page="village" class="nav-link flex items-center px-3 py-1.5 text-xs text-gray-200 hover:bg-primary-600 hover:text-white" role="menuitem">${getIcon('village', 'w-4 h-4 mr-2')}${translate('menu_village')}</a>
                                   <a href="#" data-page="equipment" class="nav-link flex items-center px-3 py-1.5 text-xs text-gray-200 hover:bg-primary-600 hover:text-white" role="menuitem">${getIcon('equipment', 'w-4 h-4 mr-2')}${translate('menu_equipment')}</a>
                                   <a href="#" data-page="inventory" class="nav-link flex items-center px-3 py-1.5 text-xs text-gray-200 hover:bg-primary-600 hover:text-white" role="menuitem">${getIcon('inventory', 'w-4 h-4 mr-2')}${translate('menu_inventory')}</a>
                                   <a href="#" data-page="profile" class="nav-link flex items-center px-3 py-1.5 text-xs text-gray-200 hover:bg-primary-600 hover:text-white" role="menuitem">${getIcon('profile', 'w-4 h-4 mr-2')}${translate('menu_profile')}</a>
                                   <a href="#" data-page="encyclopedia" class="nav-link flex items-center px-3 py-1.5 text-xs text-gray-200 hover:bg-primary-600 hover:text-white" role="menuitem">${getIcon('bookOpen', 'w-4 h-4 mr-2')}${translate('menu_encyclopedia')}</a>
                                   <a href="#" data-page="save_load" class="nav-link flex items-center px-3 py-1.5 text-xs text-gray-200 hover:bg-primary-600 hover:text-white" role="menuitem">${getIcon('save', 'w-4 h-4 mr-2')}${translate('menu_save_load')}</a>
                               </div>
                           </div>
                        </div>
                    </div>
                </div>
            </nav>

            <div class="container mx-auto px-4 py-6 flex-grow">
                 <main id="mainView" class="card" role="main" aria-live="polite">
                    ${translate('menu_select_option')}
                </main>
            </div>

            <div id="alertContainer" class="fixed bottom-5 right-5 space-y-2 z-[100]" aria-live="assertive"></div>
        </div>
    `;

    document.getElementById('logoutButton')?.addEventListener('click', () => {
        authLogout();
        navigateToLogin();
    });

    const menuButton = document.getElementById('menuButton') as HTMLButtonElement;
    const navDropdown = document.getElementById('navDropdown') as HTMLElement;

    menuButton.addEventListener('click', () => {
        const isExpanded = menuButton.getAttribute('aria-expanded') === 'true';
        menuButton.setAttribute('aria-expanded', (!isExpanded).toString());
        navDropdown.classList.toggle('hidden');
    });

    document.addEventListener('click', (event) => {
        if (!menuButton.contains(event.target as Node) && !navDropdown.contains(event.target as Node)) {
            menuButton.setAttribute('aria-expanded', 'false');
            navDropdown.classList.add('hidden');
        }
    });

    document.querySelectorAll('.nav-link').forEach(link => {
        link.addEventListener('click', (e) => {
            e.preventDefault();
            const page = link.getAttribute('data-page');
            if (page) {
                loadMainView(page);
            }
            menuButton.setAttribute('aria-expanded', 'false');
            navDropdown.classList.add('hidden');
        });
    });

    loadMainView('explore');
}

export function updateGameUI() {
    if (!currentUser || !document.getElementById('userLevel')) return;

    (document.getElementById('userLevel') as HTMLElement).textContent = currentUser.level.toString();
    (document.getElementById('userXP') as HTMLElement).textContent = currentUser.xp.toLocaleString();
    (document.getElementById('userGold') as HTMLElement).textContent = currentUser.gold.toLocaleString();

    const xpBar = document.getElementById('xpBar') as HTMLElement;
    const xpToNextLevelEl = document.getElementById('xpToNextLevel') as HTMLElement;

    const currentLevelBaseXP = XP_THRESHOLDS[currentUser.level - 1] || 0;
    const nextLevelThresholdXP = XP_THRESHOLDS[currentUser.level];

    if (nextLevelThresholdXP !== undefined && currentUser.level < XP_THRESHOLDS.length) {
      xpToNextLevelEl.textContent = ` / ${nextLevelThresholdXP.toLocaleString()}`;
      const xpIntoCurrentLevel = currentUser.xp - currentLevelBaseXP;
      const xpNeededForThisLevel = nextLevelThresholdXP - currentLevelBaseXP;
      const percentage = xpNeededForThisLevel > 0 ? (xpIntoCurrentLevel / xpNeededForThisLevel) * 100 : 0;
      xpBar.style.width = `${Math.min(100, Math.max(0,percentage))}%`;
      xpBar.setAttribute('aria-valuenow', currentUser.xp.toString());
      xpBar.setAttribute('aria-valuemin', currentLevelBaseXP.toString());
      xpBar.setAttribute('aria-valuemax', nextLevelThresholdXP.toString());
    } else {
      xpToNextLevelEl.textContent = ` / ${translate('max_level_indicator')}`;
      xpBar.style.width = '100%';
      xpBar.setAttribute('aria-valuenow', currentUser.xp.toString());
      xpBar.setAttribute('aria-valuemin', currentLevelBaseXP.toString());
      xpBar.setAttribute('aria-valuemax', currentUser.xp.toString());
    }

    const avatarImg = document.querySelector('nav img') as HTMLImageElement;
    if (avatarImg) avatarImg.src = currentUser.avatar;
}

export async function loadMainView(page: string, targetUsername?: string) {
    const mainView = document.getElementById('mainView');
    if (!mainView || !currentUser) return;

    mainView.innerHTML = `<div class="text-center py-10"><p class="text-gray-400 animate-pulse">${translate('loading_text')}</p></div>`;

    if (targetUsername && targetUsername !== currentUser.username) {
        await loadUserProfileView(targetUsername);
        return;
    }

    switch (page) {
        case 'explore':
            mainView.innerHTML = renderExplorationPage(currentUser);
            initExplorationSystem(currentUser);
            break;
        case 'village':
            mainView.innerHTML = renderVillagePage(currentUser);
            initVillageSystem(currentUser);
            break;
        case 'equipment':
            mainView.innerHTML = renderEquipmentPage(currentUser, true);
            initEquipmentSystem(currentUser, true);
            break;
        case 'inventory':
            mainView.innerHTML = renderInventoryPage(currentUser);
            initInventorySystem(currentUser);
            break;
        case 'profile':
            mainView.innerHTML = renderProfilePage(currentUser, true);
            initProfileSystem(currentUser, true);
            break;
        case 'save_load':
            mainView.innerHTML = renderSaveLoadPage(currentUser);
            initSaveLoadSystem(currentUser);
            break;
        case 'encyclopedia':
            mainView.innerHTML = renderEncyclopediaPage(currentUser);
            initEncyclopediaSystem(currentUser);
            break;
        default:
            mainView.innerHTML = `<p>${translate('welcome_message', currentUser.username)} ${translate('menu_select_option')}</p>`;
    }
}

export async function loadUserProfileView(usernameToView: string) {
    const mainView = document.getElementById('mainView');
    const localCurrentUser = getCurrentGameUser();
    if (!mainView || !localCurrentUser) return;

    mainView.innerHTML = `<div class="text-center py-10"><p class="text-gray-400 animate-pulse">${translate('loading_text')}</p></div>`;

    const targetUser = await loadUserData(usernameToView);
    if (!targetUser) {
        displayAlert(translate('user_not_found_for_view', usernameToView), 'danger');
        mainView.innerHTML = `<p>${translate('user_not_found_for_view', usernameToView)}</p>`;
        return;
    }

    const isOwn = localCurrentUser.username === targetUser.username;

    let combinedHtml = `<div class="space-y-8">`;
    combinedHtml += `<h2 class="text-3xl font-semibold text-primary-400 mb-6 text-center">${translate('user_profile_title', targetUser.username)}</h2>`;

    combinedHtml += `<section aria-labelledby="profile-details-${targetUser.username}">${renderProfilePage(targetUser, isOwn)}</section>`;

    combinedHtml += `<hr class="border-gray-700 my-6">`;
    combinedHtml += `<h3 class="text-2xl font-semibold text-primary-400 mb-4 text-center">${translate('user_equipment_title', targetUser.username)}</h3>`;
    combinedHtml += `<section aria-labelledby="equipment-details-${targetUser.username}">${renderEquipmentPage(targetUser, isOwn)}</section>`;

    combinedHtml += `</div>`;

    mainView.innerHTML = combinedHtml;

    initProfileSystem(targetUser, isOwn);
    initEquipmentSystem(targetUser, isOwn);
}


export function navigateToLogin() {
    renderLoginPage();
}

export function navigateToGame() {
    renderGamePage();
}


async function initializeApp() {
    const sessionUser = await checkSession();
    if (sessionUser) {
        setCurrentUser(sessionUser);
        navigateToGame();
    } else {
        navigateToLogin();
    }
}

initializeApp();